@@ -147,7 +147,7 @@ public class Container extends Component
   /**
    * Returns the component at the specified index.
    *
-   * @param index The index of the component to retrieve.
+   * @param n The index of the component to retrieve.
    *
    * @return The requested component.
    *
@@ -231,7 +231,7 @@ public class Container extends Component
    * Adds the specified component to this container at the end of the
    * component list.
    *
-   * @param component The component to add to the container.
+   * @param comp The component to add to the container.
    *
    * @return The same component that was added.
    */
@@ -247,7 +247,7 @@ public class Container extends Component
    * <code>add(Component, Object)</code>.
    *
    * @param name The name of the component to be added.
-   * @param component The component to be added.
+   * @param comp The component to be added.
    *
    * @return The same component that was added.
    *
@@ -263,7 +263,7 @@ public class Container extends Component
    * Adds the specified component to this container at the specified index
    * in the component list.
    *
-   * @param component The component to be added.
+   * @param comp The component to be added.
    * @param index The index in the component list to insert this child
    * at, or -1 to add at the end of the list.
    *
@@ -282,7 +282,7 @@ public class Container extends Component
    * component list.  The layout manager will use the specified constraints
    * when laying out this component.
    *
-   * @param component The component to be added to this container.
+   * @param comp The component to be added to this container.
    * @param constraints The layout constraints for this component.
    */
   public void add(Component comp, Object constraints)
@@ -295,7 +295,7 @@ public class Container extends Component
    * in the component list.  The layout manager will use the specified
    * constraints when layout out this component.
    *
-   * @param component The component to be added.
+   * @param comp The component to be added.
    * @param constraints The layout constraints for this component.
    * @param index The index in the component list to insert this child
    * at, or -1 to add at the end of the list.
@@ -314,7 +314,7 @@ public class Container extends Component
    * method.  Any subclass doing this must call the superclass version of
    * this method in order to ensure proper functioning of the container.
    *
-   * @param component The component to be added.
+   * @param comp The component to be added.
    * @param constraints The layout constraints for this component, or
    * <code>null</code> if there are no constraints.
    * @param index The index in the component list to insert this child
@@ -387,11 +387,14 @@ public class Container extends Component
               layoutMgr.addLayoutComponent(null, comp);
           }
 
-        // Post event to notify of adding the container.
-        ContainerEvent ce = new ContainerEvent(this,
-                                               ContainerEvent.COMPONENT_ADDED,
-                                               comp);
-        getToolkit().getSystemEventQueue().postEvent(ce);
+        if (isShowing ())
+          {
+            // Post event to notify of adding the component.
+            ContainerEvent ce = new ContainerEvent(this,
+                                                   ContainerEvent.COMPONENT_ADDED,
+                                                   comp);
+            getToolkit().getSystemEventQueue().postEvent(ce);
+          }
       }
   }
 
@@ -419,11 +422,14 @@ public class Container extends Component
 
         r.parent = null;
 
-        // Post event to notify of adding the container.
-        ContainerEvent ce = new ContainerEvent(this,
-                                               ContainerEvent.COMPONENT_REMOVED,
-                                               r);
-        getToolkit().getSystemEventQueue().postEvent(ce);
+        if (isShowing ())
+          {
+            // Post event to notify of removing the component.
+            ContainerEvent ce = new ContainerEvent(this,
+                                                   ContainerEvent.COMPONENT_REMOVED,
+                                                   r);
+            getToolkit().getSystemEventQueue().postEvent(ce);
+          }
       }
   }
 
@@ -706,7 +712,7 @@ public class Container extends Component
    * a superclass method so that lightweight components are properly
    * drawn.
    *
-   * @param graphics The graphics context for this paint job.
+   * @param g The graphics context for this paint job.
    */
   public void paint(Graphics g)
   {
@@ -726,7 +732,7 @@ public class Container extends Component
    * a superclass method so that lightweight components are properly
    * drawn.
    *
-   * @param graphics The graphics context for this update.
+   * @param g The graphics context for this update.
    */
   public void update(Graphics g)
   {
@@ -740,7 +746,7 @@ public class Container extends Component
    * a superclass method so that lightweight components are properly
    * drawn.
    *
-   * @param graphics The graphics context for this print job.
+   * @param g The graphics context for this print job.
    */
   public void print(Graphics g)
   {
@@ -751,7 +757,7 @@ public class Container extends Component
   /**
    * Paints all of the components in this container.
    *
-   * @param graphics The graphics context for this paint job.
+   * @param g The graphics context for this paint job.
    */
   public void paintComponents(Graphics g)
   {
@@ -762,7 +768,7 @@ public class Container extends Component
   /**
    * Prints all of the components in this container.
    *
-   * @param graphics The graphics context for this print job.
+   * @param g The graphics context for this print job.
    */
   public void printComponents(Graphics g)
   {
@@ -776,9 +782,9 @@ public class Container extends Component
    *
    * @param listener The listener to add.
    */
-  public synchronized void addContainerListener(ContainerListener l)
+  public synchronized void addContainerListener(ContainerListener listener)
   {
-    containerListener = AWTEventMulticaster.add(containerListener, l);
+    containerListener = AWTEventMulticaster.add(containerListener, listener);
   }
 
   /**
@@ -787,9 +793,9 @@ public class Container extends Component
    *
    * @param listener The listener to remove.
    */
-  public synchronized void removeContainerListener(ContainerListener l)
+  public synchronized void removeContainerListener(ContainerListener listener)
   {
-    containerListener = AWTEventMulticaster.remove(containerListener, l);
+    containerListener = AWTEventMulticaster.remove(containerListener, listener);
   }
 
   /**
@@ -825,7 +831,7 @@ public class Container extends Component
    * <code>ContainerEvent</code>, otherwise it calls the superclass
    * method.
    *
-   * @param event The event to be processed.
+   * @param e The event to be processed.
    */
   protected void processEvent(AWTEvent e)
   {
@@ -839,7 +845,7 @@ public class Container extends Component
    * Called when a container event occurs if container events are enabled.
    * This method calls any registered listeners.
    *
-   * @param event The event that occurred.
+   * @param e The event that occurred.
    */
   protected void processContainerEvent(ContainerEvent e)
   {
@@ -860,7 +866,7 @@ public class Container extends Component
   /**
    * AWT 1.0 event processor.
    *
-   * @param event The event that occurred.
+   * @param e The event that occurred.
    *
    * @deprecated use {@link #dispatchEvent(AWTEvent)} instead
    */
@@ -905,7 +911,8 @@ public class Container extends Component
    * unless the point does not exist within this container, in which
    * case <code>null</code> is returned.
    *
-   * @param point The point to return the component at.
+   * @param x The x position of the point to return the component at.
+   * @param y The y position of the point to return the component at.
    *
    * @return The component containing the specified point, or <code>null</code>
    * if there is no such point.
@@ -941,7 +948,7 @@ public class Container extends Component
    * unless the point does not exist within this container, in which
    * case <code>null</code> is returned.
    *
-   * @param point The point to return the component at.
+   * @param p The point to return the component at.
    * @return The component containing the specified point, or <code>null</code>
    * if there is no such point.
    */
@@ -1017,7 +1024,7 @@ public class Container extends Component
    * Tests whether or not the specified component is contained within
    * this components subtree.
    *
-   * @param component The component to test.
+   * @param comp The component to test.
    *
    * @return <code>true</code> if this container is an ancestor of the
    * specified component, <code>false</code> otherwise.
@@ -1056,7 +1063,7 @@ public class Container extends Component
    * Writes a listing of this container to the specified stream starting
    * at the specified indentation point.
    *
-   * @param stream The <code>PrintStream</code> to write to.
+   * @param out The <code>PrintStream</code> to write to.
    * @param indent The indentation point.
    */
   public void list(PrintStream out, int indent)
@@ -1073,7 +1080,7 @@ public class Container extends Component
    * Writes a listing of this container to the specified stream starting
    * at the specified indentation point.
    *
-   * @param stream The <code>PrintWriter</code> to write to.
+   * @param out The <code>PrintWriter</code> to write to.
    * @param indent The indentation point.
    */
   public void list(PrintWriter out, int indent)
