@@ -80,6 +80,11 @@ class EventDispatchThread extends Thread
           if (!manager.dispatchEvent (evt))
             queue.dispatchEvent(evt);
 	}
+        catch (ThreadDeath death)
+        {
+          // If someone wants to kill us, let them.
+          return;
+        }
 	catch (InterruptedException ie)
 	{
 	  // We are interrupted when we should finish executing
