@@ -1,44 +1,46 @@
 /* ICC_Profile.java -- color space profiling
    Copyright (C) 2000, 2002, 2004 Free Software Foundation
 
-   This file is part of GNU Classpath.
+This file is part of GNU Classpath.
 
-   GNU Classpath is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+GNU Classpath is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
 
-   GNU Classpath is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+GNU Classpath is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Classpath; see the file COPYING.  If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.
+You should have received a copy of the GNU General Public License
+along with GNU Classpath; see the file COPYING.  If not, write to the
+Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+02111-1307 USA.
 
-   Linking this library statically or dynamically with other modules is
-   making a combined work based on this library.  Thus, the terms and
-   conditions of the GNU General Public License cover the whole
-   combination.
+Linking this library statically or dynamically with other modules is
+making a combined work based on this library.  Thus, the terms and
+conditions of the GNU General Public License cover the whole
+combination.
+
+As a special exception, the copyright holders of this library give you
+permission to link this library with independent modules to produce an
+executable, regardless of the license terms of these independent
+modules, and to copy and distribute the resulting executable under
+terms of your choice, provided that you also meet, for each linked
+independent module, the terms and conditions of the license of that
+module.  An independent module is a module which is not derived from
+or based on this library.  If you modify this library, you may extend
+this exception to your version of the library, but you are not
+obligated to do so.  If you do not wish to do so, delete this
+exception statement from your version. */
 
-   As a special exception, the copyright holders of this library give you
-   permission to link this library with independent modules to produce an
-   executable, regardless of the license terms of these independent
-   modules, and to copy and distribute the resulting executable under
-   terms of your choice, provided that you also meet, for each linked
-   independent module, the terms and conditions of the license of that
-   module.  An independent module is a module which is not derived from
-   or based on this library.  If you modify this library, you may extend
-   this exception to your version of the library, but you are not
-   obligated to do so.  If you do not wish to do so, delete this
-   exception statement from your version. */
 
 package java.awt.color;
 
 import gnu.java.awt.color.ProfileHeader;
 import gnu.java.awt.color.TagEntry;
+
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
@@ -53,7 +55,6 @@ import java.nio.ByteBuffer;
 import java.util.Enumeration;
 import java.util.Hashtable;
 
-
 /**
  * ICC Profile - represents an ICC Color profile.
  * The ICC profile format is a standard file format which maps the transform
@@ -173,7 +174,6 @@ public class ICC_Profile implements Serializable
   public static final int icSigMeasurementTag = 0x6D656173; // 'meas' 
   public static final int icSigMediaBlackPointTag = 0x626B7074; // 'bkpt' 
   public static final int icSigMediaWhitePointTag = 0x77747074; // 'wtpt' 
-  public static final int icSigNamedColorTag = 0x6E636f6C; // ''ncol'  
   public static final int icSigNamedColor2Tag = 0x6E636C32; // 'ncl2' 
   public static final int icSigOutputResponseTag = 0x72657370; // 'resp' 
   public static final int icSigPreview0Tag = 0x70726530; // 'pre0' 
@@ -239,7 +239,7 @@ public class ICC_Profile implements Serializable
   /**
    * @serial
    */
-  private final int iccProfileSerializedDataVersion = 1;
+  private static final int iccProfileSerializedDataVersion = 1;
 
   /**
    * Constants related to generating profiles for
@@ -316,8 +316,6 @@ public class ICC_Profile implements Serializable
    */
   protected void finalize()
   {
-    header = null;
-    tagTable = null;
   }
 
   /**
