@@ -279,6 +279,9 @@ public class DataInputStream extends FilterInputStream implements DataInput
    */
   public final void readFully (byte[] buf, int offset, int len) throws IOException
   {
+    if (len < 0)
+      throw new IndexOutOfBoundsException("Negative length: " + len);
+    
     while (len > 0)
       {
 	// in.read will block until some data is available.
@@ -648,7 +651,7 @@ public class DataInputStream extends FilterInputStream implements DataInput
    *
    * @see DataInput#readUTF
    */
-  public final static String readUTF (DataInput in) throws IOException
+  public static final String readUTF(DataInput in) throws IOException
   {
     final int UTFlen = in.readUnsignedShort ();
     byte[] buf = new byte [UTFlen];
