@@ -39,6 +39,7 @@ exception statement from your version. */
 
 package java.io;
 
+import gnu.classpath.SystemProperties;
 import java.net.MalformedURLException;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -67,7 +68,7 @@ public class File implements Serializable, Comparable
    * contains the value of the <code>file.separator</code> system property.
    * An example separator string would be "/" on the GNU system.
    */
-  public static final String separator = System.getProperty("file.separator");
+  public static final String separator = SystemProperties.getProperty("file.separator");
   private static final String dupSeparator = separator + separator;
 
   /**
@@ -84,7 +85,7 @@ public class File implements Serializable, Comparable
    * the <code>path.separator</code> system property.
    */
   public static final String pathSeparator
-    = System.getProperty("path.separator");
+    = SystemProperties.getProperty("path.separator");
   
   /**
    * This is the first character of the string used to separate the host name
@@ -144,25 +145,10 @@ public class File implements Serializable, Comparable
     if (! VMFile.exists(path))
       return false;
 
-    if (!VMFile.isDirectory(path))
-      return VMFile.canWrite(path);
+    if (VMFile.isDirectory(path))
+      return VMFile.canWriteDirectory(this);
     else
-      try
-        {
-          /* If the separator is '\' a DOS-style-filesystem is assumed
-             and a short name is used, otherwise use a long name.
-             WARNING: some implementation of DOS-style-filesystems also
-             accept '/' as separator. In that case the following code
-             will fail.
-          */
-          String filename = (separatorChar!='\\')?"test-dir-write":"tst";
-  	  File test = createTempFile(filename, null, this);
-  	  return (test != null && test.delete());
-        }
-      catch (IOException ioe)
-        {
-  	  return false;
-        }
+      return VMFile.canWrite(path);
   }
 
   /**
