@@ -101,14 +101,6 @@ public class OutputStreamWriter extends Writer
     throws UnsupportedEncodingException
   {
     this.out = EncodingManager.getEncoder (out, encoding_scheme);
-    try
-    {
-      this.out.setBadCharValue('?');
-    }
-    catch(CharConversionException x)
-    {
-      // what can we do here?
-    }
   }
 
   /**
@@ -120,14 +112,6 @@ public class OutputStreamWriter extends Writer
   public OutputStreamWriter (OutputStream out)
   {
     this.out = EncodingManager.getEncoder (out);
-    try
-    {
-      this.out.setBadCharValue('?');
-    }
-    catch(CharConversionException x)
-    {
-      // what can we do here?
-    }
   }
 
   /**
@@ -199,7 +183,7 @@ public class OutputStreamWriter extends Writer
   /**
    * This method writes a single character to the output stream.
    *
-   * @param c The char to write, passed as an int.
+   * @param ch The char to write, passed as an int.
    *
    * @exception IOException If an error occurs
    */
