@@ -129,7 +129,14 @@ public final class Class implements Serializable
   Class(Object vmdata, ProtectionDomain pd)
   {
     this.vmdata = vmdata;
-    this.pd = pd;
+    // If the VM didn't supply a protection domain and the class is an array,
+    // we "inherit" the protection domain from the component type class. This
+    // saves the VM from having to worry about protection domains for array
+    // classes.
+    if (pd == null && isArray())
+      this.pd = getComponentType().pd;
+    else
+      this.pd = pd;
   }
 
   /**
@@ -188,7 +195,7 @@ public final class Class implements Serializable
     if (classloader == null)
       {
         // Check if we may access the bootstrap classloader
-        SecurityManager sm = System.getSecurityManager();
+        SecurityManager sm = SecurityManager.current;
         if (sm != null)
           {
             // Get the calling class and classloader
@@ -268,7 +275,7 @@ public final class Class implements Serializable
 
     ClassLoader loader = VMClass.getClassLoader(this);
     // Check if we may get the classloader
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
         // Get the calling class and classloader
@@ -1161,7 +1168,7 @@ public final class Class implements Serializable
    */
   public ProtectionDomain getProtectionDomain()
   {
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       sm.checkPermission(new RuntimePermission("getProtectionDomain"));
 
@@ -1213,7 +1220,8 @@ public final class Class implements Serializable
         }
     else
       {
-        status = ClassLoader.systemClassAssertionStatus.get(getName());
+        status = ClassLoader.StaticData.
+                    systemClassAssertionStatus.get(getName());
         if (status != null)
           return status.equals(Boolean.TRUE);
       }
@@ -1237,11 +1245,13 @@ public final class Class implements Serializable
       {
         String name = getPackagePortion(getName());
         if ("".equals(name))
-          status = ClassLoader.systemPackageAssertionStatus.get(null);
+          status = ClassLoader.StaticData.
+                    systemPackageAssertionStatus.get(null);
         else
           do
             {
-              status = ClassLoader.systemPackageAssertionStatus.get(name);
+              status = ClassLoader.StaticData.
+                        systemPackageAssertionStatus.get(name);
               name = getPackagePortion(name);
             }
           while (! "".equals(name) && status == null);
@@ -1297,7 +1307,7 @@ public final class Class implements Serializable
    */
   private void memberAccessCheck(int which)
   {
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
 	sm.checkMemberAccess(this, which);
