@@ -38,6 +38,7 @@ exception statement from your version. */
 
 package java.lang;
 
+import gnu.classpath.SystemProperties;
 import gnu.java.util.DoubleEnumeration;
 import gnu.java.util.EmptyEnumeration;
 
@@ -150,16 +151,72 @@ public abstract class ClassLoader
    */
   private final boolean initialized;
 
-  /**
-   * The default protection domain, used when defining a class with a null
-   * paramter for the domain.
-   */
-  static final ProtectionDomain defaultProtectionDomain;
-  static
+  static class StaticData
   {
-    CodeSource cs = new CodeSource(null, null);
-    PermissionCollection perm = Policy.getPolicy().getPermissions(cs);
-    defaultProtectionDomain = new ProtectionDomain(cs, perm);
+    /**
+     * The System Class Loader (a.k.a. Application Class Loader). The one
+     * returned by ClassLoader.getSystemClassLoader.
+     */
+    static final ClassLoader systemClassLoader =
+                              VMClassLoader.getSystemClassLoader();
+    static
+    {
+      // Find out if we have to install a default security manager. Note that
+      // this is done here because we potentially need the system class loader
+      // to load the security manager and note also that we don't need the
+      // security manager until the system class loader is created.
+      // If the runtime chooses to use a class loader that doesn't have the
+      // system class loader as its parent, it is responsible for setting
+      // up a security manager before doing so.
+      String secman = SystemProperties.getProperty("java.security.manager");
+      if (secman != null && SecurityManager.current == null)
+        {
+          if (secman.equals("") || secman.equals("default"))
+	    {
+	      SecurityManager.current = new SecurityManager();
+	    }
+	  else
+	    {
+	      try
+	        {
+	  	  Class cl = Class.forName(secman, false, StaticData.systemClassLoader);
+		  SecurityManager.current = (SecurityManager)cl.newInstance();
+	        }
+	      catch (Exception x)
+	        {
+		  throw (InternalError)
+		      new InternalError("Unable to create SecurityManager")
+		  	  .initCause(x);
+	        }
+	    }
+        }
+    }
+
+    /**
+     * The default protection domain, used when defining a class with a null
+     * parameter for the domain.
+     */
+    static final ProtectionDomain defaultProtectionDomain;
+    static
+    {
+        CodeSource cs = new CodeSource(null, null);
+        PermissionCollection perm = Policy.getPolicy().getPermissions(cs);
+        defaultProtectionDomain = new ProtectionDomain(cs, perm);
+    }
+    /**
+     * The command-line state of the package assertion status overrides. This
+     * map is never modified, so it does not need to be synchronized.
+     */
+    // Package visible for use by Class.
+    static final Map systemPackageAssertionStatus
+      = VMClassLoader.packageAssertionStatus();
+    /**
+     * The command-line state of the class assertion status overrides. This
+     * map is never modified, so it does not need to be synchronized.
+     */
+    // Package visible for use by Class.
+    static final Map systemClassAssertionStatus
+      = VMClassLoader.classAssertionStatus();
   }
 
   /**
@@ -169,14 +226,6 @@ public abstract class ClassLoader
   // Package visible for use by Class.
   boolean defaultAssertionStatus = VMClassLoader.defaultAssertionStatus();
 
-  /**
-   * The command-line state of the package assertion status overrides. This
-   * map is never modified, so it does not need to be synchronized.
-   */
-  // Package visible for use by Class.
-  static final Map systemPackageAssertionStatus
-    = VMClassLoader.packageAssertionStatus();
-
   /**
    * The map of package assertion status overrides, or null if no package
    * overrides have been specified yet. The values of the map should be
@@ -186,14 +235,6 @@ public abstract class ClassLoader
   // Package visible for use by Class.
   Map packageAssertionStatus;
 
-  /**
-   * The command-line state of the class assertion status overrides. This
-   * map is never modified, so it does not need to be synchronized.
-   */
-  // Package visible for use by Class.
-  static final Map systemClassAssertionStatus
-    = VMClassLoader.classAssertionStatus();
-
   /**
    * The map of class assertion status overrides, or null if no class
    * overrides have been specified yet. The values of the map should be
@@ -211,7 +252,7 @@ public abstract class ClassLoader
    */
   protected ClassLoader() throws SecurityException
   {
-    this(System.systemClassLoader);
+    this(StaticData.systemClassLoader);
   }
 
   /**
@@ -231,7 +272,7 @@ public abstract class ClassLoader
   protected ClassLoader(ClassLoader parent)
   {
     // May we create a new classloader?
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       sm.checkCreateClassLoader();
     this.parent = parent;
@@ -426,7 +467,7 @@ public abstract class ClassLoader
     throws ClassFormatError
   {
     if (domain == null)
-      domain = defaultProtectionDomain;
+      domain = StaticData.defaultProtectionDomain;
     if (! initialized)
       throw new SecurityException("attempt to define class from uninitialized class loader");
     Class retval = VMClassLoader.defineClass(this, name, data,
@@ -460,7 +501,7 @@ public abstract class ClassLoader
   protected final Class findSystemClass(String name)
     throws ClassNotFoundException
   {
-    return Class.forName(name, false, System.systemClassLoader);
+    return Class.forName(name, false, StaticData.systemClassLoader);
   }
 
   /**
@@ -476,7 +517,7 @@ public abstract class ClassLoader
   public final ClassLoader getParent()
   {
     // Check if we may return the parent classloader.
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
         Class c = VMSecurityManager.getClassContext()[1];
@@ -622,7 +663,7 @@ public abstract class ClassLoader
    */
   public static final URL getSystemResource(String name)
   {
-    return System.systemClassLoader.getResource(name);
+    return StaticData.systemClassLoader.getResource(name);
   }
 
   /**
@@ -638,7 +679,7 @@ public abstract class ClassLoader
    */
   public static Enumeration getSystemResources(String name) throws IOException
   {
-    return System.systemClassLoader.getResources(name);
+    return StaticData.systemClassLoader.getResources(name);
   }
 
   /**
@@ -719,16 +760,16 @@ public abstract class ClassLoader
   public static ClassLoader getSystemClassLoader()
   {
     // Check if we may return the system classloader
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
 	Class c = VMSecurityManager.getClassContext()[1];
 	ClassLoader cl = c.getClassLoader();
-	if (cl != null && cl != System.systemClassLoader)
+	if (cl != null && cl != StaticData.systemClassLoader)
 	  sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
 
-    return System.systemClassLoader;
+    return StaticData.systemClassLoader;
   }
 
   /**
@@ -887,7 +928,7 @@ public abstract class ClassLoader
   {
     if (packageAssertionStatus == null)
       packageAssertionStatus
-        = new HashMap(systemPackageAssertionStatus);
+        = new HashMap(StaticData.systemPackageAssertionStatus);
     packageAssertionStatus.put(name, Boolean.valueOf(enabled));
   }
   
@@ -907,7 +948,8 @@ public abstract class ClassLoader
                                                    boolean enabled)
   {
     if (classAssertionStatus == null)
-      classAssertionStatus = new HashMap(systemClassAssertionStatus);
+      classAssertionStatus = 
+        new HashMap(StaticData.systemClassAssertionStatus);
     // The toString() hack catches null, as required.
     classAssertionStatus.put(name.toString(), Boolean.valueOf(enabled));
   }
@@ -947,7 +989,7 @@ public abstract class ClassLoader
 
   private static URL[] getExtClassLoaderUrls()
   {
-    String classpath = getSystemProperty("java.ext.dirs", "");
+    String classpath = SystemProperties.getProperty("java.ext.dirs", "");
     StringTokenizer tok = new StringTokenizer(classpath, File.pathSeparator);
     ArrayList list = new ArrayList();
     while (tok.hasMoreTokens())
@@ -983,7 +1025,7 @@ public abstract class ClassLoader
 
   private static URL[] getSystemClassLoaderUrls()
   {
-    String classpath = getSystemProperty("java.class.path", ".");
+    String classpath = SystemProperties.getProperty("java.class.path", ".");
     StringTokenizer tok = new StringTokenizer(classpath, File.pathSeparator, true);
     ArrayList list = new ArrayList();
     while (tok.hasMoreTokens())
@@ -1021,7 +1063,7 @@ public abstract class ClassLoader
 		boolean resolve)
 		throws ClassNotFoundException
 	    {
-		SecurityManager sm = Runtime.securityManager;
+		SecurityManager sm = SecurityManager.current;
 		if (sm != null)
 		{
 		    int lastDot = name.lastIndexOf('.');
@@ -1031,7 +1073,7 @@ public abstract class ClassLoader
 		return super.loadClass(name, resolve);
 	    }
 	};
-    String loader = getSystemProperty("java.system.class.loader", null);
+    String loader = SystemProperties.getProperty("java.system.class.loader", null);
     if (loader == null)
       {
 	return systemClassLoader;
@@ -1050,15 +1092,4 @@ public abstract class ClassLoader
 		.initCause(e);
       }
   }
-
-  static String getSystemProperty(String name, String defaultValue)
-  {
-    // access properties directly to bypass security
-    String val = System.properties.getProperty(name);
-    if (val == null)
-      {
-	val = defaultValue;
-      }
-    return val;
-  }
 }
