@@ -243,7 +243,7 @@ public final class Double extends Number implements Comparable
    * @see #NEGATIVE_INFINITY
    * @since 1.2
    */
-  public static native double parseDouble(String s);
+  public static native double parseDouble(String str);
 
   /**
    * Return <code>true</code> if the <code>double</code> has the same
@@ -392,9 +392,9 @@ public final class Double extends Number implements Comparable
    * <code>Double.NaN</code> as equal, but treats <code>0.0</code> and
    * <code>-0.0</code> as unequal.
    *
-   * <p>Note that <code>d1.equals(d2)<code> is identical to
+   * <p>Note that <code>d1.equals(d2)</code> is identical to
    * <code>doubleToLongBits(d1.doubleValue()) ==
-   *    doubleToLongBits(d2.doubleValue())<code>.
+   *    doubleToLongBits(d2.doubleValue())</code>.
    *
    * @param obj the object to compare
    * @return whether the objects are semantically equal
