@@ -227,7 +227,7 @@ public class InetAddress implements Serializable
   {
     // Mask against high order bits of 1110
     if (addr.length == 4)
-      return (addr[0] & 0xF0) == 0xE0;
+      return (addr[0] & 0xf0) == 0xe0;
 
     return false;
   }
@@ -253,7 +253,7 @@ public class InetAddress implements Serializable
   {
     // This is the IPv4 implementation.
     // Any class derived from InetAddress should override this.
-    return addr[0] == 0x7F;
+    return (addr[0] & 0xff) == 0x7f;
   }
 
   /**
@@ -278,18 +278,17 @@ public class InetAddress implements Serializable
   {
     // This is the IPv4 implementation.
     // Any class derived from InetAddress should override this.
+
     // 10.0.0.0/8
-    if (addr[0] == 0x0A)
+    if ((addr[0] & 0xff) == 0x0a)
       return true;
 
-    // XXX: Suns JDK 1.4.1 (on Linux) seems to have a bug here:
-    // it says 172.16.0.0 - 172.255.255.255 are site local addresses
     // 172.16.0.0/12
-    if (addr[0] == 0xAC && (addr[1] & 0xF0) == 0x01)
+    if ((addr[0] & 0xff) == 0xac && (addr[1] & 0xf0) == 0x10)
       return true;
 
     // 192.168.0.0/16
-    if (addr[0] == 0xC0 && addr[1] == 0xA8)
+    if ((addr[0] & 0xff) == 0xc0 && (addr[1] & 0xff) == 0xa8)
       return true;
 
     // XXX: Do we need to check more addresses here ?
@@ -310,7 +309,7 @@ public class InetAddress implements Serializable
   }
 
   /**
-   * Utility reoutine to check if InetAddress is a node local multicast address
+   * Utility routine to check if InetAddress is a node local multicast address.
    *
    * @since 1.4
    */
@@ -323,7 +322,7 @@ public class InetAddress implements Serializable
   }
 
   /**
-   * Utility reoutine to check if InetAddress is a link local multicast address
+   * Utility routine to check if InetAddress is a link local multicast address.
    *
    * @since 1.4
    */
@@ -334,11 +333,13 @@ public class InetAddress implements Serializable
     if (! isMulticastAddress())
       return false;
 
-    return (addr[0] == 0xE0 && addr[1] == 0x00 && addr[2] == 0x00);
+    return ((addr[0] & 0xff) == 0xe0
+	    && (addr[1] & 0xff)  == 0x00
+	    && (addr[2] & 0xff)  == 0x00);
   }
 
   /**
-   * Utility routine to check if InetAddress is a site local multicast address
+   * Utility routine to check if InetAddress is a site local multicast address.
    *
    * @since 1.4
    */
@@ -351,8 +352,8 @@ public class InetAddress implements Serializable
   }
 
   /**
-   * Utility reoutine to check if InetAddress is a organization local
-   * multicast address
+   * Utility routine to check if InetAddress is a organization local
+   * multicast address.
    *
    * @since 1.4
    */
@@ -407,8 +408,14 @@ public class InetAddress implements Serializable
       }
 
     // Try to find the FDQN now
-    // FIXME: This does not work with IPv6.
-    InetAddress address = new Inet4Address(getAddress(), null);
+    InetAddress address;
+    byte[] ipaddr = getAddress();
+
+    if (ipaddr.length == 16)
+      address = new Inet6Address(getAddress(), null);
+    else
+      address = new Inet4Address(getAddress(), null);
+
     return address.getHostName();
   }
 
@@ -468,7 +475,7 @@ public class InetAddress implements Serializable
     int i = len > 4 ? len - 4 : 0;
 
     for (; i < len; i++)
-      hash = (hash << 8) | (addr[i] & 0xFF);
+      hash = (hash << 8) | (addr[i] & 0xff);
 
     return hash;
   }
@@ -796,7 +803,7 @@ public class InetAddress implements Serializable
    * This method returns the hostname for a given IP address.  It will
    * throw an UnknownHostException if the hostname cannot be determined.
    *
-   * @param ip The IP address as a int array
+   * @param ip The IP address as a byte array
    *
    * @return The hostname
    *
@@ -841,7 +848,7 @@ public class InetAddress implements Serializable
     int i = len - 4;
 
     for (; i < len; i++)
-      address = address << 8 | (((int) addr[i]) & 0xFF);
+      address = address << 8 | (addr[i] & 0xff);
 
     oos.defaultWriteObject();
   }
