@@ -327,7 +327,7 @@ final class ByteBufferHelper
 
   public static void putDouble (ByteBuffer buffer, double value, ByteOrder order)
   {
-    putLong (buffer, Double.doubleToLongBits (value), order);
+    putLong (buffer, Double.doubleToRawLongBits (value), order);
   }
   
   public static double getDouble (ByteBuffer buffer, int index, ByteOrder order)
@@ -338,6 +338,7 @@ final class ByteBufferHelper
   public static void putDouble (ByteBuffer buffer, int index,
 				double value, ByteOrder order)
   {
-    putLong (buffer, index, Double.doubleToLongBits (value), order);
+    putLong (buffer, index, Double.doubleToRawLongBits (value), order);
   }
 } // ByteBufferHelper
+
