@@ -112,6 +112,8 @@ final class ByteBufferImpl extends ByteBuffer
 
   public ByteBuffer compact ()
   {
+    checkIfReadOnly();
+    mark = -1;
     int pos = position();
     if (pos > 0)
       {
@@ -120,6 +122,11 @@ final class ByteBufferImpl extends ByteBuffer
 	position(count);
 	limit(capacity());
       }
+    else
+      {
+	position(limit());
+	limit(capacity());
+      }
     return this;
   }
   
