@@ -116,6 +116,11 @@ final class IntViewBufferImpl extends IntBuffer
         position (count);
         limit (capacity ());
       }
+    else
+      {
+	position(limit());
+	limit(capacity());
+      }
     return this;
   }
   
