@@ -137,7 +137,7 @@ public abstract class DateFormat extends Format implements Cloneable
     public static final DateFormat.Field TIME_ZONE
 	= new Field("timezone", Calendar.ZONE_OFFSET);
  
-    public static final DateFormat.Field[] allFields =
+    static final DateFormat.Field[] allFields =
     {
       ERA, YEAR, MONTH, DAY_OF_MONTH, HOUR_OF_DAY1,
       HOUR_OF_DAY0, MINUTE, SECOND, MILLISECOND,
@@ -288,12 +288,9 @@ public abstract class DateFormat extends Format implements Cloneable
    * This method returns a list of available locales supported by this
    * class.
    */
-  public static Locale[] getAvailableLocales ()
+  public static Locale[] getAvailableLocales()
   {
-    // FIXME
-    Locale[] l = new Locale[1];
-    l[0] = Locale.US;
-    return l;
+    return Locale.getAvailableLocales();
   }
 
   /**
