@@ -130,7 +130,7 @@ public class TreeMap extends AbstractMap
   /**
    * The root node of this TreeMap.
    */
-  private transient Node root = nil;
+  private transient Node root;
 
   /**
    * The size of this TreeMap. Package visible for use by nested classes.
@@ -213,6 +213,7 @@ public class TreeMap extends AbstractMap
   public TreeMap(Comparator c)
   {
     comparator = c;
+    fabricateTree(0);
   }
 
   /**
@@ -851,7 +852,11 @@ public class TreeMap extends AbstractMap
   private void fabricateTree(final int count)
   {
     if (count == 0)
-      return;
+      {
+	root = nil;
+	size = 0;
+	return;
+      }
 
     // We color every row of nodes black, except for the overflow nodes.
     // I believe that this is the optimal arrangement. We construct the tree
@@ -1531,7 +1536,7 @@ public class TreeMap extends AbstractMap
      * @param key the key to check
      * @return true if the key is in range
      */
-    final boolean keyInRange(Object key)
+    boolean keyInRange(Object key)
     {
       return ((minKey == nil || compare(key, minKey) >= 0)
               && (maxKey == nil || compare(key, maxKey) < 0));
