@@ -59,12 +59,8 @@ import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.Set;
 
-/**
- *
- */
 public final class Subject implements Serializable
 {
-
   // Fields.
   // -------------------------------------------------------------------------
 
@@ -81,8 +77,8 @@ public final class Subject implements Serializable
    */
   private boolean readOnly;
 
-  private transient final SecureSet pubCred;
-  private transient final SecureSet privCred;
+  private final transient SecureSet pubCred;
+  private final transient SecureSet privCred;
 
   // Constructors.
   // -------------------------------------------------------------------------
@@ -371,7 +367,6 @@ public final class Subject implements Serializable
    */
   private static class SecureSet extends AbstractSet implements Serializable
   {
-
     // Fields.
     // -----------------------------------------------------------------------
 
@@ -383,7 +378,7 @@ public final class Subject implements Serializable
 
     private final Subject subject;
     private final LinkedList elements;
-    private transient final int type;
+    private final transient int type;
 
     // Constructors.
     // -----------------------------------------------------------------------
