@@ -35,6 +35,7 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing;
 
 import java.awt.BorderLayout;
@@ -44,10 +45,10 @@ import java.awt.Dimension;
 import java.awt.LayoutManager;
 import java.awt.LayoutManager2;
 import java.io.Serializable;
+
 import javax.accessibility.AccessibleRole;
 import javax.swing.plaf.RootPaneUI;
 
-
 /**
  * This class is where JComponents are added to. Unlike awt where you could
  * just say frame.add(), with swing you need to say frame.getRootPane()
@@ -309,6 +310,16 @@ public class JRootPane extends JComponent
   /** DOCUMENT ME! */
   private static final long serialVersionUID = 8690748000348575668L;
 
+  public static final int NONE = 0;
+  public static final int FRAME = 1;
+  public static final int PLAIN_DIALOG = 2;
+  public static final int INFORMATION_DIALOG = 3;
+  public static final int ERROR_DIALOG = 4;
+  public static final int COLOR_CHOOSER_DIALOG = 5;
+  public static final int FILE_CHOOSER_DIALOG = 6;
+  public static final int QUESTION_DIALOG = 7;
+  public static final int WARNING_DIALOG = 8;
+	  
   /** DOCUMENT ME! */
   protected Component glassPane;
 
@@ -321,6 +332,13 @@ public class JRootPane extends JComponent
   /** DOCUMENT ME! */
   protected Container contentPane;
 
+  protected JButton defaultButton;
+
+  /**
+   * @since 1.4
+   */
+  private int windowDecorationStyle = NONE;
+  
   /**
    * DOCUMENT ME!
    *
@@ -332,6 +350,14 @@ public class JRootPane extends JComponent
     getLayeredPane().add(menuBar, JLayeredPane.FRAME_CONTENT_LAYER);
   }
 
+  /**
+   * @deprecated Replaced by <code>setJMenuBar()</code>
+   */
+  public void setMenuBar(JMenuBar m)
+  {
+    setJMenuBar(m);
+  }
+
   /**
    * DOCUMENT ME!
    *
@@ -342,6 +368,14 @@ public class JRootPane extends JComponent
     return menuBar;
   }
 
+  /**
+   * @deprecated Replaced by <code>getJMenuBar()</code>
+   */
+  public JMenuBar getMenuBar()
+  {
+    return getJMenuBar();
+  }
+
   /**
    * DOCUMENT ME!
    *
@@ -469,7 +503,7 @@ public class JRootPane extends JComponent
    *
    * @return DOCUMENT ME!
    */
-  protected JComponent createContentPane()
+  protected Container createContentPane()
   {
     JPanel p = new JPanel();
     p.setName(this.getName() + ".contentPane");
@@ -541,4 +575,47 @@ public class JRootPane extends JComponent
   {
     return "RootPaneUI";
   }
+
+  public JButton getDefaultButton()
+  {
+    return defaultButton;
+  }
+  
+  public void setDefaultButton(JButton newButton)
+  {
+    if (defaultButton == newButton)
+      return;
+    
+    JButton oldButton = defaultButton;
+    defaultButton = newButton;
+    firePropertyChange("defaultButton", oldButton, newButton);
+  }
+
+  /**
+   * @since 1.4
+   */
+  public int getWindowDecorationStyle()
+  {
+    return windowDecorationStyle;
+  }
+
+  /**
+   * @since 1.4
+   */
+  public void setWindowDecorationStyle(int style)
+  {
+    if (style != NONE
+	&& style != FRAME
+	&& style != INFORMATION_DIALOG
+	&& style != ERROR_DIALOG
+	&& style != COLOR_CHOOSER_DIALOG
+	&& style != FILE_CHOOSER_DIALOG
+	&& style != QUESTION_DIALOG
+	&& style != WARNING_DIALOG)
+      throw new IllegalArgumentException("invalid style");
+    
+    int oldStyle = windowDecorationStyle;
+    windowDecorationStyle = style;
+    firePropertyChange("windowDecorationStyle", oldStyle, style);
+  }
 }
