@@ -45,12 +45,10 @@ import java.awt.Insets;
 import java.awt.Point;
 import java.awt.Rectangle;
 
-import javax.accessibility.Accessible;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 import javax.swing.plaf.ViewportUI;
 
-
 /**
  *  
  * <pre>
@@ -129,6 +127,7 @@ public class JViewport extends JComponent
   public JViewport()
   {
     setOpaque(true);
+    setScrollMode(BLIT_SCROLL_MODE);
     updateUI();
   }
 
@@ -214,11 +213,17 @@ public class JViewport extends JComponent
                          getExtentSize());
   }
 
+  /**
+   * @deprecated 1.4
+   */
   public boolean isBackingStoreEnabled()
   {
     return scrollMode == BACKINGSTORE_SCROLL_MODE;
   }
 
+  /**
+   * @deprecated 1.4
+   */
   public void setBackingStoreEnabled(boolean b)
   {
     if (b && scrollMode != BACKINGSTORE_SCROLL_MODE)
@@ -253,9 +258,9 @@ public class JViewport extends JComponent
       remove(0);
     if (v != null)
       {
-    add(v);
-    fireStateChanged();
-  }
+        add(v);
+        fireStateChanged();
+      }
   }
 
   public void revalidate()
@@ -305,23 +310,11 @@ public class JViewport extends JComponent
     return false;
   }
 
-  public ChangeListener[] getChangeListeners() 
-  {
-    return (ChangeListener[]) getListeners(ChangeListener.class);
-  }
-
   public void paint(Graphics g)
   {
     paintComponent(g);
   }
 
-  void fireStateChanged()
-  {
-    ChangeListener[] listeners = getChangeListeners();
-    for (int i = 0; i < listeners.length; ++i)
-      listeners[i].stateChanged(changeEvent);
-  }
-
   public void addChangeListener(ChangeListener listener)
   {
     listenerList.add(ChangeListener.class, listener);
@@ -332,6 +325,18 @@ public class JViewport extends JComponent
     listenerList.remove(ChangeListener.class, listener);
   }
 
+  public ChangeListener[] getChangeListeners() 
+  {
+    return (ChangeListener[]) getListeners(ChangeListener.class);
+  }
+
+  protected void fireStateChanged()
+  {
+    ChangeListener[] listeners = getChangeListeners();
+    for (int i = 0; i < listeners.length; ++i)
+      listeners[i].stateChanged(changeEvent);
+  }
+
   /**
    * This method returns the String ID of the UI class of  Separator.
    *
@@ -349,4 +354,24 @@ public class JViewport extends JComponent
   {
     setUI((ViewportUI) UIManager.getUI(this));
   }            
+
+  /**
+   * This method returns the viewport's UI delegate.
+   *
+   * @return The viewport's UI delegate.
+   */
+  public ViewportUI getUI()
+  {
+    return (ViewportUI) ui;
+  }
+
+  /**
+   * This method sets the viewport's UI delegate.
+   *
+   * @param ui The viewport's UI delegate.
+   */
+  public void setUI(ViewportUI ui)
+  {
+    super.setUI(ui);
+  }
 }
