@@ -80,6 +80,7 @@ import javax.swing.plaf.ComponentUI;
  * UI Delegate for JComboBox
  *
  * @author Olga Rodimina
+ * @author Robert Schuster
  */
 public class BasicComboBoxUI extends ComboBoxUI
 {
@@ -783,22 +784,25 @@ public class BasicComboBoxUI extends ComboBoxUI
       {
 	Object currentValue = comboBox.getSelectedItem();
 	boolean isPressed = arrowButton.getModel().isPressed();
-	if (currentValue != null)
-	  {
-	    Component comp = comboBox.getRenderer()
+
+	/* Gets the component to be drawn for the current value.
+	 * If there is currently no selected item we will take an empty
+	 * String as replacement.
+	 */
+	Component comp = comboBox.getRenderer()
 	                             .getListCellRendererComponent(listBox,
-	                                                           currentValue,
+	                                                           (currentValue != null ? currentValue : ""),
 	                                                           -1,
 	                                                           isPressed,
 	                                                           hasFocus);
-	    if (! comboBox.isEnabled())
+	if (! comboBox.isEnabled())
 	      comp.setEnabled(false);
 
-	    g.translate(borderInsets.left, borderInsets.top);
+	g.translate(borderInsets.left, borderInsets.top);
 	    comp.setBounds(0, 0, bounds.width, bounds.height);
 	    comp.paint(g);
 	    g.translate(-borderInsets.left, -borderInsets.top);
-	  }
+	    
 	comboBox.revalidate();
       }
     else
