@@ -36,30 +36,32 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package java.lang;
 
-import java.security.ProtectionDomain;
-import java.net.URL;
-import java.net.MalformedURLException;
-import java.io.IOException;
+import gnu.classpath.SystemProperties;
+import gnu.java.lang.SystemClassLoader;
+import gnu.java.util.EmptyEnumeration;
+
 import java.io.File;
+import java.io.IOException;
+import java.lang.reflect.Constructor;
+import java.net.MalformedURLException;
+import java.net.URL;
+import java.security.ProtectionDomain;
 import java.util.Enumeration;
 import java.util.Map;
 import java.util.HashMap;
 import java.util.StringTokenizer;
 import java.util.Vector;
-import java.lang.reflect.Constructor;
-import gnu.java.lang.SystemClassLoader;
-
-import gnu.java.util.EmptyEnumeration;
 
 /**
  * java.lang.VMClassLoader is a package-private helper for VMs to implement
  * on behalf of java.lang.ClassLoader.
  *
  * @author John Keiser
- * @author Mark Wielaard <mark@klomp.org>
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Mark Wielaard (mark@klomp.org)
+ * @author Eric Blake (ebb9@email.byu.edu)
  */
 final class VMClassLoader
 {
@@ -161,7 +163,7 @@ final class VMClassLoader
   static Enumeration getResources(String name)
   {
     StringTokenizer st = new StringTokenizer(
-      ClassLoader.getSystemProperty("java.boot.class.path", "."),
+      SystemProperties.getProperty("java.boot.class.path", "."),
       File.pathSeparator);
     Vector v = new Vector();
     while (st.hasMoreTokens())
